/*
	File:		FireWireMPEG.h

    Synopsis: This is the top level header file for the FireWireMPEG framework. This
    is the only header that needs to be included to use the clases and functions in
    the framework.
 
	Copyright: 	 Copyright 2001-2003 Apple Computer, Inc. All rights reserved.

	Written by: ayanowitz

 Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
 ("Apple") in consideration of your agreement to the following terms, and your
 use, installation, modification or redistribution of this Apple software
 constitutes acceptance of these terms.  If you do not agree with these terms,
 please do not use, install, modify or redistribute this Apple software.

 In consideration of your agreement to abide by the following terms, and subject
 to these terms, Apple grants you a personal, non-exclusive license, under Apples
 copyrights in this original Apple software (the "Apple Software"), to use,
 reproduce, modify and redistribute the Apple Software, with or without
 modifications, in source and/or binary forms; provided that if you redistribute
 the Apple Software in its entirety and without modifications, you must retain
 this notice and the following text and disclaimers in all such redistributions of
 the Apple Software.  Neither the name, trademarks, service marks or logos of
 Apple Computer, Inc. may be used to endorse or promote products derived from the
 Apple Software without specific prior written permission from Apple.  Except as
 expressly stated in this notice, no other rights or licenses, express or implied,
 are granted by Apple herein, including but not limited to any patent rights that
 may be infringed by your derivative works or by other works in which the Apple
 Software may be incorporated.

 The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
 WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
 COMBINATION WITH YOUR PRODUCTS.

 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
 OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
 (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
 ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __FIREWIREMPEG_FIREWIREMPEG__
#define __FIREWIREMPEG_FIREWIREMPEG__

// Sizes of MPEG2 stuff
#define kMPEG2XmitCIPOnlySize 8
#define kMPEG2TSPacketSize 188
#define kMPEG2TSPacketSizeInWords 47
#define kMPEG2SourcePacketSize 192
#define kMPEG2DataBlocksPerPacket 8

// Some Max data rate constants for
// different number of packets per cycle
#define kMaxDataRate_EighthTSPacketPerCycle 	1504000
#define kMaxDataRate_QuarterTSPacketPerCycle 	3008000
#define kMaxDataRate_HalfTSPacketPerCycle 		6016000
#define kMaxDataRate_OneTSPacketPerCycle 		12032000
#define kMaxDataRate_TwoTSPackestPerCycle 		24064000
#define kMaxDataRate_ThreeTSPacketsPerCycle 	36096000
#define kMaxDataRate_FourTSPacketsPerCycle 		48128000
#define kMaxDataRate_FiveTSPacketsPerCycle 		60160000

// A define for any available isoch channel.
// Specify this value as the MPEG2 receiver
// or transmitter channel num, to allow the FireWire
// family's isoch code determine and allocate an available
// channel. An application can then determine which
// channel was allocated in the message callback
// for the AllocateChannel message.
// Note: This value should only be used if the
// MPEG2Transmitter or MPEG2Recevier are created
// with the doIRMAllocations flag set to true.
#define kAnyAvailableIsochChannel 0xFFFFFFFF

// Include Required IOKit Headers
#include <IOKit/IOKitLib.h>
#include <IOKit/firewire/IOFireWireLibIsoch.h>

// Include all the rest of this framework's headers
#include "StringLogger.h"
#include "TSPacket.h"
#include "PSITables.h"
#include "MPEG2XmitCycle.h"
#include "MPEG2Transmitter.h"
#include "MPEG2Receiver.h"

//////////////////////////////////////////////////////////////////////////////////
//
// Prototypes for FireWireMPEG object creation/destruction helper functions.
// These functions create/prepare or destroy both a FireWireMPEG class object, 
// as well as a dedicated real-time thread for the object's callbacks and DCL
// processing.
//
//////////////////////////////////////////////////////////////////////////////////

// Create and setup a MPEG2Receiver object and it's dedicated thread
IOReturn CreateMPEG2Receiver(MPEG2Receiver **ppReceiver,
							 DataPushProc dataPushProcHandler,
							 MPEG2ReceiverMessageProc messageProcHandler = nil,
							 StringLogger *stringLogger = nil,
							 IOFireWireLibNubRef nubInterface = nil,
							 unsigned int cyclesPerSegment = kCyclesPerReceiveSegment,
							 unsigned int numSegments = kNumReceiveSegments,
							 bool doIRMAllocations = false);

// Destroy a MPEG2Receiver object created with CreateMPEG2Receiver(), and it's dedicated thread
IOReturn DestroyMPEG2Receiver(MPEG2Receiver *pReceiver);

// Create and setup a MPEG2Transmitter object and it's dedicated thread
IOReturn CreateMPEG2Transmitter(MPEG2Transmitter **ppTransmitter,
								DataPullProc dataPullProcHandler,
								MPEG2TransmitterMessageProc messageHandler = nil,
								StringLogger *stringLogger = nil,
								IOFireWireLibNubRef nubInterface = nil,
								unsigned int cyclesPerSegment = kCyclesPerTransmitSegment,
								unsigned int numSegments = kNumTransmitSegments,
								bool doIRMAllocations = false,
								unsigned int packetsPerCycle = kNumTSPacketsPerCycle);

// Destroy a MPEG2Transmitter object created with CreateMPEG2Transmitter(), and it's dedicated thread
IOReturn DestroyMPEG2Transmitter(MPEG2Transmitter *pTransmitter);

#endif // __FIREWIREMPEG_FIREWIREMPEG__